#!/usr/bin/env python

import sys
import numpy as np
from scipy import stats, integrate
import matplotlib.pyplot as plt
import seaborn as sns
sns.set(color_codes=True)

if len(sys.argv) < 2:
    print("Specify trace output filename!")
    sys.exit(1)

fd = open(sys.argv[1], "r")

# in TM1, we have four ZMQ pop before the UHD push
# here we analyse the time between last pop and the push
#
# format:
#92821997,ZMQ,pop
#92849179,UHD,push 27

eti_pop_to_uhd_push = []
eti_time_zmq_pop = 0
eti_time_uhd_push = 0

uhd_wait_to_pop = []
uhd_time_uhd_wait = 0
uhd_time_uhd_pop = 0

for line in fd:
    try:
        timestr, source, event = line.split(",")
    except ValueError:
        break

    if source == "ZMQ" and event.startswith("pop"):
        eti_time_zmq_pop = int(timestr)
    elif source == "UHD" and event.startswith("push"):
        eti_time_uhd_push = int(timestr)
    elif source == "UHD" and event.startswith("wait"):
        uhd_time_uhd_wait = int(timestr)
    elif source == "UHD" and event.startswith("pop"):
        uhd_time_uhd_pop = int(timestr)

    if eti_time_zmq_pop > 0 and eti_time_uhd_push > 0:
        eti_pop_to_uhd_push.append(eti_time_uhd_push - eti_time_zmq_pop)
        eti_time_zmq_pop = 0
        eti_time_uhd_push = 0

    if uhd_time_uhd_pop > 0 and uhd_time_uhd_wait > 0:
        uhd_wait_to_pop.append(uhd_time_uhd_pop - uhd_time_uhd_wait)
        uhd_time_uhd_wait = 0
        uhd_time_uhd_pop = 0


#sns.distplot(eti_pop_to_uhd_push, bins=120)
#sns.distplot(uhd_wait_to_pop, bins=120)
#sns.lmplot(eti_pop_to_uhd_push)
plt.plot(uhd_wait_to_pop)
plt.plot(eti_pop_to_uhd_push)
plt.legend(["UHD wait time","MOD duration"])
plt.show()
